#!/bin/bash
VER=1.2

#--[ Intro ]---------------------------------------------------#
#                                                              #
# Tur-Links. An addon to Tur-Space (works fine alone too).     #
#                                                              #
# This script will create symlinks to all your releases in a   #
# central dir somewhere. Its useful if you have, for example,  #
# your DIVX's spread out over multiple harddrives but still    #
# wish to provide the means for users to see all of them in    #
# the same dir.                                                #
#                                                              #
#--[ Installation ]--------------------------------------------#
#                                                              #
# Copy this script to /glftpd/bin. Chmod it to 700 or similar. #
#                                                              #
# Change the settings follows:                                 #
#                                                              #
# TULS     = Full path to "tuls". A required binary. If you    #
#            dont have this one, its available at my webpage   #
#            at www.grandis.nu/glftpd                          #
#            Make sure its working before continuing.          #
#                                                              #
# GLROOT   = The path to your glftpd dir. Usually just /glftpd #
#                                                              #
# SECTIONS = Here is what you specify from where to symlink &  #
#            where to put those symlinks.                      #
#            Seperate sources (where to look) with a : and the #
#            final location should have a * infront of it.     #
#                                                              #
#            Example:                                          #
#            /site/1:/site/2:/site/3*/site/All                 #
#                                                              #
#            The above example would put symlinks from the 3   #
#            directories /site/1, /site/2 and /site/3 into     #
#            the /site/All directory.                          #
#                                                              #
#            This all have to be on one line. You can no use   #
#            a newline to seperate this.                       #
#            However, you can add as many of these as you like #
#            and if so, seperate them with a newline. Example: #
#                                                              #
#            SECTIONS="                                        #
#            /site/1:/site/2:/site/3*/site/All_Mp3             #
#            /site/A:/site/B:/site/C*/site/All_0day            #
#            "                                                 #
#                                                              #
#            You can use more then one newline to clean it up  #
#            if that helps (see example).                      #
#                                                              #
#            New in V1.2. You can now use DEEP, 2xDEEP and     #
#            3xDEEP to automatically go through subdirs.       #
#            So, instead of:                                   #
#            /site/Archive/DVDR/DVDR1:/site/Archive/DVDR/DVDR2 #
#            you can use:                                      #
#            /site/Archive/DVDR~DEEP                           #
#            and it will automatically go through every dir in #
#            /site/Archive/DVDR                                #
#                                                              #
#            So, DEEP will enter 1 subdir.                     #
#            2xDEEP enters 2 subdirs (does not add anything    #
#            from the first subdir).                           #
#            3xDEEP enters 3 subdirs down.                     #
#                                                              #
#            Now, you can NOT use the same destination dir for #
#            more then one section. This is because it will    #
#            remove every symlink it finds in the destination  #
#            dir before it starts. This ensures that there are #
#            no dead symlinks.                                 #
#                                                              #
# EXCLUDE  = Any dirs you do not want to symlink.              #
#            Normally, you'd use ^ and $ to specify start and  #
#            end, but since we use tuls to read dirs, you use  #
#            : to specify start and end instead.               #
#                                                              #
#            Like the example: ":GROUPS:" means it will not    #
#            create symlinks if the dir is called exactly      #
#            GROUPS. If you were you specify only "GROUPS", it #
#            would not create a symlink if the dir contained   #
#            the word GROUPS somewhere in it.                  #
#                                                              #
#            Likewise, specifying "GROUPS:" means it must END  #
#            with GROUPS only.                                 #
#                                                              #
#            If using ~DEEP, ~2xDEEP or ~3xDEEP, this is also  #
#            used for specifying which dirs it will NOT enter. #
#                                                              #
#            Seperate exclusions with a | (see example).       #
#                                                              #
#--[ Running ]-------------------------------------------------#
#                                                              #
# Specify 'debug' as first argument to see what its doing.     #
#                                                              #
# I run this every 45 minutes in the crontab. Why so often?    #
# Its fairly quick and then I know the symlinks are up to date.#
# To crontab it to run every 45 minutes past the hour:         #
# 45 * * * *      /glftpd/bin/tur-links.sh                     #
#                                                              #
#--[ Notes ]---------------------------------------------------#
#                                                              #
# There is no way to get the symlinks to be the "correct" date.#
# I've tried but can not get it to do that. Cant even 'touch'  #
# a symlink with the correct date.                             #
#                                                              #
# Doing it like this opens another option for you as mentioned #
# by nm0. Mount all the seperate disks in /Archive/DISKS or so #
# and then hide it with hidden_files in glftpd.conf.           #
# Make symlinks from them to a central dir, like /Archive/MP3. #
# Now, the users does not even see the seperate disks and      #
# instead see it all as one big section. Atleast until they    #
# click on a symlink =).                                       #
#                                                              #
#--[ Changelog ]-----------------------------------------------#
#                                                              #
# 1.2   : Now uses 'unlink' to remove old symlinks before each #
#         new run.                                             #
#                                                              #
#         You can now use ~DEEP, ~2xDEEP or ~3xDEEP to         #
#         automatically enter subdirs, instead of specifying   #
#         each one seperatly.                                  #
#                                                              #
#         'debug' mode added. Specify debug as first and only  #
#         only argument when running the script to see what    #
#         its doing.                                           #
#                                                              #
# 1.1   : It wont try to create the symlink if there already   #
#         is one with the same name. Usually get an error when #
#         any spacemaker script runs at the same time, thus    #
#         the release exists in 2 locations at once.           #
#                                                              #
#--[ Configuration ]-------------------------------------------#

TULS=/glftpd/bin/tuls

GLROOT=/glftpd


SECTIONS="

/site/DIVX:/site/Archive/DIVX/DIVX1:/site/Archive/DIVX/DIVX2*/site/Archive/DIVX/All

/site/DVDR:/site/Archive/DVDR/DVDR1:/site/Archive/DVDR/DVDR2*/site/Archive/DVDR/All

" # <- end of SECTIONS. Dont touch.

# Example for simpler SECTIONS. Note the ~DEEP at the end of one sourcepath:
# SECTIONS="
# /site/DIVX:/site/Archive/DIVX~DEEP*/site/Archive/DIVX/All
# "


EXCLUDE=":GROUPS:|:_PRE:|:PRE:|:lost\+found:|:mountpoint:"


#--[ Script Start ]--------------------------------------------#

if [ ! -x "$TULS" ]; then
  echo "Error. $TULS could not be executed or dosnt exist."
  exit 1
fi
if [ ! -d "$GLROOT" ]; then
  echo "Error. $GLROOT was not found or is not a directory."
  exit 1
fi

proc_symclear() {
  cd "$CLEARHERE"
  for symlink in `$TULS | grep "^l" | awk -F"::::" '{print $4}'`; do
    unlink "$symlink"
  done
}

proc_findem() {
  added="0"
  for release in `$TULS | grep "^d" | egrep -iv "$EXCLUDE|:\.\.:|:\.\:" | awk -F"::::" '{print $4}'`; do
    if [ ! -h "$GLROOT$TO/$release" ]; then
      ln -s "$SOURCEDIR/$release" "$GLROOT$TO/$release"
      if [ "$DEBUG" = "TRUE" ]; then
        added=$[added+1]
      fi
    else
      proc_debug "Skipping $SOURCEDIR/$release - Symlink already exists."
    fi
  done
}

if [ "$1" = "debug" ]; then
  DEBUG="TRUE"
fi

proc_debug() {
  if [ "$DEBUG" = "TRUE" ]; then
    echo "$*"
  fi
}

## Go !
for section in $SECTIONS; do
  proc_debug ""
  proc_debug "-------------------------------------------"
  proc_debug "Starting work on $section"

  FROMRAW="`echo "$section" | cut -d '*' -f1`"
  TO="`echo "$section" | cut -d '*' -f2`"
  proc_debug "Working FROM: $FROMRAW"
  proc_debug "Working TO  : $TO"

  if [ -z "$FROMRAW" ]; then
    echo "Error. Did not get any 'from' location from $section"
    exit 1
  elif [ -z "$TO" ]; then
    echo "Error. Did not get any 'to' location from $section"
    exit 1
  fi

  FROMRAW="`echo "$FROMRAW" | tr ':' ' '`"

  ## Clean out old symlinks.
  CLEARHERE="$GLROOT$TO"
  proc_debug "Clean Syms  : $CLEARHERE"
  proc_symclear

  for FROM in $FROMRAW; do

    DEEPMODE="`echo "$FROM" | cut -d '~' -f2`"
    if [ "$DEEPMODE" ]; then
      FROM="`echo "$FROM" | cut -d '~' -f1`"
    fi
    proc_debug "Starting on : $FROM"

    if [ "$DEEPMODE" ]; then

      case $DEEPMODE in
        DEEP) cd "$GLROOT$FROM"
              proc_debug "DEEPMODE    : $DEEPMODE - $GLROOT$FROM/*/"
              for subdir1 in `$TULS | grep "^d" | egrep -iv "$EXCLUDE|:\.\.:|:\.\:" | awk -F"::::" '{print $4}'`; do
                 proc_debug "Ent Subdir  : $subdir1"
                 SOURCEDIR="$FROM/$subdir1"
                 cd "$GLROOT$SOURCEDIR"
                 proc_findem
                 proc_debug "Done. Added : $added"
                 proc_debug ""
              done

              ;;
        2xDEEP) cd "$GLROOT$FROM"
                proc_debug "DEEPMODE    : $DEEPMODE - $GLROOT$FROM/*/*/"
                for subdir1 in `$TULS | grep "^d" | egrep -iv "$EXCLUDE|:\.\.:|:\.\:" | awk -F"::::" '{print $4}'`; do
                  cd "$GLROOT$FROM/$subdir1"
                  proc_debug ""
                  proc_debug "Ent Subdir1 : $subdir1"
                  for subdir2 in `$TULS | grep "^d" | egrep -iv "$EXCLUDE|:\.\.:|:\.\:" | awk -F"::::" '{print $4}'`; do
                    proc_debug "Ent Subdir2 : $subdir1/$subdir2"
                    SOURCEDIR="$FROM/$subdir1/$subdir2"
                    cd "$GLROOT$SOURCEDIR"
                    proc_findem
                    proc_debug "Done. Added : $added"
                    proc_debug ""
                  done
                done
              ;;
        3xDEEP) cd "$GLROOT$FROM"
                proc_debug "DEEPMODE    : $DEEPMODE - $GLROOT$FROM/*/*/*/"
                for subdir1 in `$TULS | grep "^d" | egrep -iv "$EXCLUDE|:\.\.:|:\.\:" | awk -F"::::" '{print $4}'`; do
                  cd "$GLROOT$FROM/$subdir1"
                  proc_debug ""
                  proc_debug "Ent Subdir1 : $subdir1"
                  for subdir2 in `$TULS | grep "^d" | egrep -iv "$EXCLUDE|:\.\.:|:\.\:" | awk -F"::::" '{print $4}'`; do
                    cd "$GLROOT$FROM/$subdir1/$subdir2"
                     proc_debug "Ent Subdir2 : $subdir1/$subdir2"
                     for subdir3 in `$TULS | grep "^d" | egrep -iv "$EXCLUDE|:\.\.:|:\.\:" | awk -F"::::" '{print $4}'`; do
                       SOURCEDIR="$FROM/$subdir1/$subdir2/$subdir3"
                       cd "$GLROOT$SOURCEDIR"
                       proc_debug "Ent Subdir3 : $subdir1/$subdir2/$subdir3"
                       proc_findem
                    done
                  done
                done
              ;;
         *) echo "Error. Unknown DEEP mode: $DEEPMODE. (DEEP, 2xDEEP, 3xDEEP)"
            exit 1
         ;;
      esac
    else
      if [ ! -d "$GLROOT$TO" ]; then
        echo "Error. $GLROOT$TO does not exist or is not a directory. Skipping."
      else
        for FROM in $FROMRAW; do
          if [ ! -d "$GLROOT$FROM" ]; then
            echo "Error. $GLROOT$FROM does not exist or is not a dir. Skipping."
          else
            SOURCEDIR="$FROM"
            cd "$GLROOT$SOURCEDIR"
            proc_findem
          fi
        done
      fi
    fi
  done
done

exit 0
